import { File, Function, Type, FunctionParams, Const } from '@kubb/react';
import { isOptional } from '@kubb/oas';
import { getComments, getPathParams } from '@kubb/plugin-oas/utils';
import { URLPath } from '@kubb/core/utils';
import { jsxs, Fragment, jsx } from '@kubb/react/jsx-runtime';

// src/components/Query.tsx
function getParams({ pathParamsType, paramsCasing, typeSchemas }) {
  return FunctionParams.factory({
    pathParams: {
      mode: pathParamsType === "object" ? "object" : "inlineSpread",
      children: getPathParams(typeSchemas.pathParams, { typed: true, casing: paramsCasing })
    },
    data: typeSchemas.request?.name ? {
      type: typeSchemas.request?.name,
      optional: isOptional(typeSchemas.request?.schema)
    } : undefined,
    params: typeSchemas.queryParams?.name ? {
      type: typeSchemas.queryParams?.name,
      optional: isOptional(typeSchemas.queryParams?.schema)
    } : undefined
  });
}
var getTransformer = ({ operation, schemas, casing }) => {
  const path = new URLPath(operation.path, { casing });
  const keys = [
    path.toObject({
      type: "path",
      stringify: true
    }),
    schemas.queryParams?.name ? "...(params ? [params] : [])" : undefined,
    schemas.request?.name ? "...(data ? [data] : [])" : undefined
  ].filter(Boolean);
  return keys;
};
function QueryKey({ name, typeSchemas, paramsCasing, pathParamsType, operation, typeName, transformer = getTransformer }) {
  const params = getParams({ pathParamsType, paramsCasing, typeSchemas });
  const keys = transformer({
    operation,
    schemas: typeSchemas,
    casing: paramsCasing
  });
  return /* @__PURE__ */ jsxs(Fragment, { children: [
    /* @__PURE__ */ jsx(File.Source, { name, isExportable: true, isIndexable: true, children: /* @__PURE__ */ jsx(Function.Arrow, { name, export: true, params: params.toConstructor(), singleLine: true, children: `[${keys.join(", ")}] as const` }) }),
    /* @__PURE__ */ jsx(File.Source, { name: typeName, isExportable: true, isIndexable: true, isTypeOnly: true, children: /* @__PURE__ */ jsx(Type, { name: typeName, export: true, children: `ReturnType<typeof ${name}>` }) })
  ] });
}
QueryKey.getParams = getParams;
QueryKey.getTransformer = getTransformer;
function getParams2({ paramsType, paramsCasing, pathParamsType, typeSchemas }) {
  if (paramsType === "object") {
    return FunctionParams.factory({
      data: {
        mode: "object",
        children: {
          ...getPathParams(typeSchemas.pathParams, { typed: true, casing: paramsCasing })
        }
      }
    });
  }
  return FunctionParams.factory({
    pathParams: typeSchemas.pathParams?.name ? {
      mode: pathParamsType === "object" ? "object" : "inlineSpread",
      children: getPathParams(typeSchemas.pathParams, { typed: true, casing: paramsCasing }),
      optional: isOptional(typeSchemas.pathParams?.schema)
    } : undefined
  });
}
function Url({ name, isExportable = true, isIndexable = true, typeSchemas, baseURL, paramsType, paramsCasing, pathParamsType, operation }) {
  const path = new URLPath(operation.path, { casing: paramsCasing });
  const params = getParams2({ paramsType, paramsCasing, pathParamsType, typeSchemas });
  return /* @__PURE__ */ jsx(File.Source, { name, isExportable, isIndexable, children: /* @__PURE__ */ jsx(Function, { name, export: isExportable, params: params.toConstructor(), children: `return ${path.toTemplateString({ prefix: baseURL })} as const` }) });
}
Url.getParams = getParams2;
function getParams3({ paramsType, paramsCasing, pathParamsType, typeSchemas }) {
  if (paramsType === "object") {
    return FunctionParams.factory({
      data: {
        mode: "object",
        children: {
          ...getPathParams(typeSchemas.pathParams, { typed: true, casing: paramsCasing }),
          data: typeSchemas.request?.name ? {
            type: typeSchemas.request?.name,
            optional: isOptional(typeSchemas.request?.schema)
          } : undefined,
          params: typeSchemas.queryParams?.name ? {
            type: typeSchemas.queryParams?.name,
            optional: isOptional(typeSchemas.queryParams?.schema)
          } : undefined,
          headers: typeSchemas.headerParams?.name ? {
            type: typeSchemas.headerParams?.name,
            optional: isOptional(typeSchemas.headerParams?.schema)
          } : undefined
        }
      },
      config: {
        type: typeSchemas.request?.name ? `Partial<RequestConfig<${typeSchemas.request?.name}>> & { client?: typeof client }` : "Partial<RequestConfig> & { client?: typeof client }",
        default: "{}"
      }
    });
  }
  return FunctionParams.factory({
    pathParams: typeSchemas.pathParams?.name ? {
      mode: pathParamsType === "object" ? "object" : "inlineSpread",
      children: getPathParams(typeSchemas.pathParams, { typed: true, casing: paramsCasing }),
      optional: isOptional(typeSchemas.pathParams?.schema)
    } : undefined,
    data: typeSchemas.request?.name ? {
      type: typeSchemas.request?.name,
      optional: isOptional(typeSchemas.request?.schema)
    } : undefined,
    params: typeSchemas.queryParams?.name ? {
      type: typeSchemas.queryParams?.name,
      optional: isOptional(typeSchemas.queryParams?.schema)
    } : undefined,
    headers: typeSchemas.headerParams?.name ? {
      type: typeSchemas.headerParams?.name,
      optional: isOptional(typeSchemas.headerParams?.schema)
    } : undefined,
    config: {
      type: typeSchemas.request?.name ? `Partial<RequestConfig<${typeSchemas.request?.name}>> & { client?: typeof client }` : "Partial<RequestConfig> & { client?: typeof client }",
      default: "{}"
    }
  });
}
function Client({
  name,
  isExportable = true,
  isIndexable = true,
  typeSchemas,
  baseURL,
  dataReturnType,
  parser,
  zodSchemas,
  paramsType,
  paramsCasing,
  pathParamsType,
  operation,
  urlName
}) {
  const path = new URLPath(operation.path, { casing: paramsCasing });
  const contentType = operation.getContentType();
  const isFormData = contentType === "multipart/form-data";
  const headers = [
    contentType !== "application/json" ? `'Content-Type': '${contentType}'` : undefined,
    typeSchemas.headerParams?.name ? "...headers" : undefined
  ].filter(Boolean);
  const TError = `ResponseErrorConfig<${typeSchemas.errors?.map((item) => item.name).join(" | ") || "Error"}>`;
  const generics = [typeSchemas.response.name, TError, typeSchemas.request?.name || "unknown"].filter(Boolean);
  const params = getParams3({ paramsType, paramsCasing, pathParamsType, typeSchemas });
  const urlParams = Url.getParams({
    paramsType,
    paramsCasing,
    pathParamsType,
    typeSchemas
  });
  const clientParams = FunctionParams.factory({
    config: {
      mode: "object",
      children: {
        method: {
          value: JSON.stringify(operation.method.toUpperCase())
        },
        url: {
          value: urlName ? `${urlName}(${urlParams.toCall()}).toString()` : path.template
        },
        baseURL: baseURL && !urlName ? {
          value: JSON.stringify(baseURL)
        } : undefined,
        params: typeSchemas.queryParams?.name ? {} : undefined,
        data: typeSchemas.request?.name ? {
          value: isFormData ? "formData" : undefined
        } : undefined,
        requestConfig: {
          mode: "inlineSpread"
        },
        headers: headers.length ? {
          value: headers.length ? `{ ${headers.join(", ")}, ...requestConfig.headers }` : undefined
        } : undefined
      }
    }
  });
  const formData = isFormData ? `
   const formData = new FormData()
   if(data) {
    Object.keys(data).forEach((key) => {
      const value = data[key as keyof typeof data];
      if (typeof key === "string" && (typeof value === "string" || value instanceof Blob)) {
        formData.append(key, value);
      }
    })
   }
  ` : "";
  return /* @__PURE__ */ jsx(File.Source, { name, isExportable, isIndexable, children: /* @__PURE__ */ jsxs(
    Function,
    {
      name,
      async: true,
      export: isExportable,
      params: params.toConstructor(),
      JSDoc: {
        comments: getComments(operation)
      },
      children: [
        "const { client:request = client, ...requestConfig } = config",
        /* @__PURE__ */ jsx("br", {}),
        /* @__PURE__ */ jsx("br", {}),
        formData,
        `const res = await request<${generics.join(", ")}>(${clientParams.toCall()})`,
        /* @__PURE__ */ jsx("br", {}),
        dataReturnType === "full" && parser === "zod" && zodSchemas && `return {...res, data: ${zodSchemas.response.name}.parse(res.data)}`,
        dataReturnType === "data" && parser === "zod" && zodSchemas && `return ${zodSchemas.response.name}.parse(res.data)`,
        dataReturnType === "full" && parser === "client" && "return res",
        dataReturnType === "data" && parser === "client" && "return res.data"
      ]
    }
  ) });
}
Client.getParams = getParams3;
function Operations({ name, operations }) {
  const operationsObject = {};
  operations.forEach((operation) => {
    operationsObject[operation.getOperationId()] = {
      path: new URLPath(operation.path).URL,
      method: operation.method
    };
  });
  return /* @__PURE__ */ jsx(File.Source, { name, isExportable: true, isIndexable: true, children: /* @__PURE__ */ jsx(Const, { name, export: true, asConst: true, children: JSON.stringify(operationsObject, undefined, 2) }) });
}
function getParams4({ paramsType, paramsCasing, pathParamsType, typeSchemas }) {
  if (paramsType === "object") {
    return FunctionParams.factory({
      data: {
        mode: "object",
        children: {
          ...getPathParams(typeSchemas.pathParams, { typed: true, casing: paramsCasing }),
          data: typeSchemas.request?.name ? {
            type: typeSchemas.request?.name,
            optional: isOptional(typeSchemas.request?.schema)
          } : undefined,
          params: typeSchemas.queryParams?.name ? {
            type: typeSchemas.queryParams?.name,
            optional: isOptional(typeSchemas.queryParams?.schema)
          } : undefined,
          headers: typeSchemas.headerParams?.name ? {
            type: typeSchemas.headerParams?.name,
            optional: isOptional(typeSchemas.headerParams?.schema)
          } : undefined
        }
      },
      config: {
        type: typeSchemas.request?.name ? `Partial<RequestConfig<${typeSchemas.request?.name}>> & { client?: typeof client }` : "Partial<RequestConfig> & { client?: typeof client }",
        default: "{}"
      }
    });
  }
  return FunctionParams.factory({
    pathParams: typeSchemas.pathParams?.name ? {
      mode: pathParamsType === "object" ? "object" : "inlineSpread",
      children: getPathParams(typeSchemas.pathParams, { typed: true, casing: paramsCasing }),
      optional: isOptional(typeSchemas.pathParams?.schema)
    } : undefined,
    data: typeSchemas.request?.name ? {
      type: typeSchemas.request?.name,
      optional: isOptional(typeSchemas.request?.schema)
    } : undefined,
    params: typeSchemas.queryParams?.name ? {
      type: typeSchemas.queryParams?.name,
      optional: isOptional(typeSchemas.queryParams?.schema)
    } : undefined,
    headers: typeSchemas.headerParams?.name ? {
      type: typeSchemas.headerParams?.name,
      optional: isOptional(typeSchemas.headerParams?.schema)
    } : undefined,
    config: {
      type: typeSchemas.request?.name ? `Partial<RequestConfig<${typeSchemas.request?.name}>> & { client?: typeof client }` : "Partial<RequestConfig> & { client?: typeof client }",
      default: "{}"
    }
  });
}
function QueryOptions({ name, clientName, typeSchemas, paramsCasing, paramsType, dataReturnType, pathParamsType, queryKeyName }) {
  const TData = dataReturnType === "data" ? typeSchemas.response.name : `ResponseConfig<${typeSchemas.response.name}>`;
  const TError = `ResponseErrorConfig<${typeSchemas.errors?.map((item) => item.name).join(" | ") || "Error"}>`;
  const params = getParams4({ paramsType, paramsCasing, pathParamsType, typeSchemas });
  const clientParams = Client.getParams({
    paramsCasing,
    typeSchemas,
    paramsType,
    pathParamsType
  });
  const queryKeyParams = QueryKey.getParams({
    pathParamsType,
    paramsCasing,
    typeSchemas
  });
  const enabled = Object.entries(queryKeyParams.flatParams).map(([key, item]) => item && !item.optional ? key : undefined).filter(Boolean).join("&& ");
  const enabledText = enabled ? `enabled: !!(${enabled}),` : "";
  return /* @__PURE__ */ jsx(File.Source, { name, isExportable: true, isIndexable: true, children: /* @__PURE__ */ jsx(Function, { name, export: true, params: params.toConstructor(), children: `
      const queryKey = ${queryKeyName}(${queryKeyParams.toCall()})
      return queryOptions<${TData}, ${TError}, ${TData}, typeof queryKey>({
      ${enabledText}
       queryKey,
       queryFn: async ({ signal }) => {
          config.signal = signal
          return ${clientName}(${clientParams.toCall()})
       },
      })
` }) });
}
QueryOptions.getParams = getParams4;
function getParams5({ paramsType, paramsCasing, pathParamsType, dataReturnType, typeSchemas }) {
  const TData = dataReturnType === "data" ? typeSchemas.response.name : `ResponseConfig<${typeSchemas.response.name}>`;
  const TError = `ResponseErrorConfig<${typeSchemas.errors?.map((item) => item.name).join(" | ") || "Error"}>`;
  if (paramsType === "object") {
    return FunctionParams.factory({
      data: {
        mode: "object",
        children: {
          ...getPathParams(typeSchemas.pathParams, { typed: true, casing: paramsCasing }),
          data: typeSchemas.request?.name ? {
            type: typeSchemas.request?.name,
            optional: isOptional(typeSchemas.request?.schema)
          } : undefined,
          params: typeSchemas.queryParams?.name ? {
            type: typeSchemas.queryParams?.name,
            optional: isOptional(typeSchemas.queryParams?.schema)
          } : undefined,
          headers: typeSchemas.headerParams?.name ? {
            type: typeSchemas.headerParams?.name,
            optional: isOptional(typeSchemas.headerParams?.schema)
          } : undefined
        }
      },
      options: {
        type: `
{
  query?: Partial<CreateBaseQueryOptions<${[TData, TError, "TData", "TQueryData", "TQueryKey"].join(", ")}>>,
  client?: ${typeSchemas.request?.name ? `Partial<RequestConfig<${typeSchemas.request?.name}>> & { client?: typeof client }` : "Partial<RequestConfig> & { client?: typeof client }"}
}
`,
        default: "{}"
      }
    });
  }
  return FunctionParams.factory({
    pathParams: typeSchemas.pathParams?.name ? {
      mode: pathParamsType === "object" ? "object" : "inlineSpread",
      children: getPathParams(typeSchemas.pathParams, { typed: true, casing: paramsCasing }),
      optional: isOptional(typeSchemas.pathParams?.schema)
    } : undefined,
    data: typeSchemas.request?.name ? {
      type: typeSchemas.request?.name,
      optional: isOptional(typeSchemas.request?.schema)
    } : undefined,
    params: typeSchemas.queryParams?.name ? {
      type: typeSchemas.queryParams?.name,
      optional: isOptional(typeSchemas.queryParams?.schema)
    } : undefined,
    headers: typeSchemas.headerParams?.name ? {
      type: typeSchemas.headerParams?.name,
      optional: isOptional(typeSchemas.headerParams?.schema)
    } : undefined,
    options: {
      type: `
{
  query?: Partial<CreateBaseQueryOptions<${[TData, TError, "TData", "TQueryData", "TQueryKey"].join(", ")}>>,
  client?: ${typeSchemas.request?.name ? `Partial<RequestConfig<${typeSchemas.request?.name}>> & { client?: typeof client }` : "Partial<RequestConfig> & { client?: typeof client }"}
}
`,
      default: "{}"
    }
  });
}
function Query({
  name,
  queryKeyTypeName,
  queryOptionsName,
  queryKeyName,
  paramsCasing,
  paramsType,
  pathParamsType,
  dataReturnType,
  typeSchemas,
  operation
}) {
  const TData = dataReturnType === "data" ? typeSchemas.response.name : `ResponseConfig<${typeSchemas.response.name}>`;
  const TError = `ResponseErrorConfig<${typeSchemas.errors?.map((item) => item.name).join(" | ") || "Error"}>`;
  const returnType = `CreateQueryResult<${["TData", TError].join(", ")}> & { queryKey: TQueryKey }`;
  const generics = [`TData = ${TData}`, `TQueryData = ${TData}`, `TQueryKey extends QueryKey = ${queryKeyTypeName}`];
  const queryKeyParams = QueryKey.getParams({
    pathParamsType,
    typeSchemas,
    paramsCasing
  });
  const queryOptionsParams = QueryOptions.getParams({
    paramsType,
    paramsCasing,
    pathParamsType,
    typeSchemas
  });
  const params = getParams5({
    paramsCasing,
    paramsType,
    pathParamsType,
    dataReturnType,
    typeSchemas
  });
  const queryOptions = `${queryOptionsName}(${queryOptionsParams.toCall()}) as unknown as CreateBaseQueryOptions`;
  return /* @__PURE__ */ jsx(File.Source, { name, isExportable: true, isIndexable: true, children: /* @__PURE__ */ jsx(
    Function,
    {
      name,
      export: true,
      generics: generics.join(", "),
      params: params.toConstructor(),
      JSDoc: {
        comments: getComments(operation)
      },
      children: `
       const { query: queryOptions, client: config = {} } = options ?? {}
       const queryKey = queryOptions?.queryKey ?? ${queryKeyName}(${queryKeyParams.toCall()})

       const query = createQuery(() => ({
        ...${queryOptions},
        queryKey,
        initialData: null,
        ...queryOptions as unknown as Omit<CreateBaseQueryOptions, "queryKey">
       })) as ${returnType}

       query.queryKey = queryKey as TQueryKey

       return query
       `
    }
  ) });
}
Query.getParams = getParams5;

export { Client, Operations, Query, QueryKey, QueryOptions, Url };
//# sourceMappingURL=chunk-6WAAM4TO.js.map
//# sourceMappingURL=chunk-6WAAM4TO.js.map